diff --git a/src/video_output/video_output.c b/src/video_output/video_output.c
index 7e27b5e..2ad4e1b 100644
--- a/src/video_output/video_output.c
+++ b/src/video_output/video_output.c
@@ -439,7 +439,7 @@ vout_thread_t * __vout_Create( vlc_object_t *p_parent, video_format_t *p_fmt )
         /* Apply video filter2 objects on the first vout */
         p_vout->p->psz_vf2 =
             var_CreateGetStringCommand( p_vout, "video-filter" );
-
+        p_vout->p->psz_current_vf2 = NULL;
         p_vout->p->b_first_vout = true;
     }
     else
@@ -459,12 +459,16 @@ vout_thread_t * __vout_Create( vlc_object_t *p_parent, video_format_t *p_fmt )
         var_Create( p_vout, "video-filter",
                     VLC_VAR_STRING | VLC_VAR_ISCOMMAND );
         p_vout->p->psz_vf2 = var_GetString( p_vout, "video-filter" );
+        p_vout->p->psz_current_vf2 = NULL;
 
         /* */
         p_vout->p->b_first_vout = false;
     }
 
+    msg_Dbg( p_vout, "p_vout->p->psz_vf2 initialized with '%s'", p_vout->p->psz_vf2);
+
     var_AddCallback( p_vout, "video-filter", VideoFilter2Callback, NULL );
+
     p_vout->p->p_vf2_chain = filter_chain_New( p_vout, "video filter2",
         false, video_filter_buffer_allocation_init, NULL, p_vout );
 
@@ -587,6 +591,8 @@ static void vout_Destructor( vlc_object_t * p_this )
     /* */
     free( p_vout->p->psz_filter_chain );
     free( p_vout->p->psz_title );
+    /* FIXME: may be we need to clean this also on close? */
+    free( p_vout->p->psz_current_vf2 );
 
     config_ChainDestroy( p_vout->p_cfg );
 
@@ -1369,15 +1375,26 @@ static void* RunThread( void *p_this )
 
             es_format_Init( &fmt, VIDEO_ES, p_vout->fmt_render.i_chroma );
             fmt.video = p_vout->fmt_render;
+
             filter_chain_Reset( p_vout->p->p_vf2_chain, &fmt, &fmt );
 
             if( filter_chain_AppendFromString( p_vout->p->p_vf2_chain,
                                                p_vout->p->psz_vf2 ) < 0 )
                 msg_Err( p_vout, "Video filter chain creation failed" );
 
+            /*
+              instead of dup - we can also reuse the same pointer?
+              and avoid strdup and free?
+            */
+            free( p_vout->p->psz_current_vf2 );
+            p_vout->p->psz_current_vf2 = strdup( p_vout->p->psz_vf2 );
+
             free( p_vout->p->psz_vf2 );
             p_vout->p->psz_vf2 = NULL;
 
+            /* is this only here to restore the original quality?
+               after changing the filter chain?
+            */
             if( i_picture_qtype_last != QTYPE_NONE )
                 PostProcessSetFilterQuality( p_vout );
         }
@@ -1635,7 +1652,14 @@ static int VideoFilter2Callback( vlc_object_t *p_this, char const *psz_cmd,
     (void)psz_cmd; (void)oldval; (void)p_data;
 
     vlc_mutex_lock( &p_vout->p->vfilter_lock );
-    p_vout->p->psz_vf2 = strdup( newval.psz_string );
+
+    if( !p_vout->p->psz_current_vf2 ||
+        !newval.psz_string ||
+        strcmp(p_vout->p->psz_current_vf2, newval.psz_string) )
+    {
+       free( p_vout->p->psz_vf2 );
+       p_vout->p->psz_vf2 = strdup( newval.psz_string );
+    }
     vlc_mutex_unlock( &p_vout->p->vfilter_lock );
 
     return VLC_SUCCESS;
